module fifo_simple_gold(/*AUTOARG*/
		    // Outputs 
		    data_out, fifo_empty, fifo_full, data_out_valid,
		    // Inputs
		    data_in, data_in_valid, pop_fifo, clk, rst
		 );

   

   input [63:0] data_in;   
   input        data_in_valid;
   input        pop_fifo;


   output [63:0] data_out;
   output        fifo_empty;
   output        fifo_full;
   output        data_out_valid;
   input 	 clk;
   input 	 rst;
   
   reg 		 fifo_full;
   reg 		 fifo_empty;
   
   reg [2:0] 	 wr_ptr;
   reg [2:0] 	 rd_ptr;
   wire [1:0] 	 wr_loc;
   wire [1:0] 	 rd_loc;
   
   reg [63:0] 	 mem [0:3];
   assign wr_loc = wr_ptr[1 : 0];
   assign rd_loc = rd_ptr[1 : 0];

   wire [63:0] DEBUG_mem_0;
   wire [63:0] DEBUG_mem_1;
   wire [63:0] DEBUG_mem_2;
   wire [63:0] DEBUG_mem_3;

   assign DEBUG_mem_0 = mem[0];
   assign DEBUG_mem_1 = mem[1];
   assign DEBUG_mem_2 = mem[2];
   assign DEBUG_mem_3 = mem[3];

   assign data_out_valid = pop_fifo ? (fifo_empty ? 1'b0 : 1'b1) :1'b0;
   assign data_out = fifo_empty ? 64'b0 : mem[rd_loc];
 
   always @(rd_ptr or wr_ptr)
     begin
       //default catch-alls
	fifo_empty <= 1'b0;
	fifo_full  <= 1'b0;
	if(rd_ptr[2-1:0]==wr_ptr[2-1:0])
	  begin
	     if(rd_ptr[2]==wr_ptr[2])
	       fifo_empty <= 1'b1;
	     else
	       fifo_full  <= 1'b1;
	     
	  end//end if
     end//end always

   always @(posedge clk)
     begin
	if ((data_in_valid == 1'b1) && (fifo_full != 1'b1))
	  mem[wr_loc] <= data_in;
     end

   always @(posedge clk) begin
       if(rst) begin
	  wr_ptr <= 'h0;
	  rd_ptr <= 'h0;
	  end // end if
       else 
	 begin
	    if(data_in_valid & (~fifo_full))
	      begin
		 wr_ptr <= wr_ptr+1;
		 
	      end
	    if(pop_fifo & (~fifo_empty))
	      rd_ptr <= rd_ptr+1;
	 end //end else
   end//end always

   always @ (posedge clk)
     $display("GOLD :: wr_ptr = %d, rd ptr = %d, full = %d, empty = %d", wr_ptr, rd_ptr, fifo_full, fifo_empty);
   
endmodule // fifo_hier
